@@ -54,8 +54,8 @@ class CarbonaraBasedStorageToozLock(object):
     def stop(self):
         self.coord.stop()
 
-    def __call__(self, metric):
-        lock_name = b"gnocchi-" + str(metric.id).encode('ascii')
+    def __call__(self, metric_id):
+        lock_name = b"gnocchi-" + str(metric_id).encode('ascii')
         return self.coord.get_lock(lock_name)
 
 
@@ -146,7 +146,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         raise NotImplementedError
 
     def delete_metric(self, metric):
-        with self._lock(metric):
+        with self._lock(metric.id):
             self._delete_metric(metric)
 
     @staticmethod
@@ -167,9 +167,17 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         deleted_metrics_id = (set(map(uuid.UUID, metrics_to_process))
                               - set(m.id for m in metrics))
         for metric_id in deleted_metrics_id:
-            self._delete_unprocessed_measures_for_metric_id(metric_id)
+            # NOTE(jd): We need to lock the metric otherwise we might delete
+            # measures that another worker might be processing. Deleting
+            # measurement files under its feet is not nice!
+            lock = self._lock(metric_id)
+            lock.acquire(blocking=sync)
+            try:
+                self._delete_unprocessed_measures_for_metric_id(metric_id)
+            finally:
+                lock.release()
         for metric in metrics:
-            lock = self._lock(metric)
+            lock = self._lock(metric.id)
             agg_methods = list(metric.archive_policy.aggregation_methods)
             # Do not block if we cannot acquire the lock, that means some other
             # worker is doing the job. We'll just ignore this metric and may
@@ -178,6 +186,13 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                 try:
                     LOG.debug("Processing measures for %s" % metric)
                     with self._process_measure_for_metric(metric) as measures:
+                        # NOTE(mnaser): The metric could have been handled by
+                        #               another worker, ignore if no measures.
+                        if len(measures) == 0:
+                            LOG.debug("Skipping %s (already processed)"
+                                      % metric)
+                            continue
+
                         try:
                             with timeutils.StopWatch() as sw:
                                 raw_measures = self._get_measures(metric,
